==> vmmagent-0.6.3-sources.jar: org/ow2/jasmine/vmm/agent/domain/policy/PackingVMPlacementPolicy.java <==
/**
 * JASMINe VMMapi: JASMINe Virtual Machine Management API
 * Copyright (C) 2009 France Telecom R&D
 * Contact: jasmine@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: PackingVMPlacementPolicy.java 3944 2009-06-12 08:09:29Z dangtran $
 * --------------------------------------------------------------------------
 */
package org.ow2.jasmine.vmm.agent.domain.policy;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;

import org.apache.log4j.Logger;
import org.ow2.jasmine.vmm.api.HostMXBean;
import org.ow2.jasmine.vmm.api.ServerPoolMXBean;
import org.ow2.jasmine.vmm.api.VMConfigSpec;

/**
 * PackingVMPlacementPolicy is a VM placement policy which attempts to pack the
 * virtual machines of a server pool on as few as physical hosts as possible.
 */
public class PackingVMPlacementPolicy implements VMPlacementPolicy {
    static Logger logger = Logger.getLogger(PackingVMPlacementPolicy.class);

    public PackingVMPlacementPolicy() {
    }

    /*
     * (non-Javadoc)
     * @see
     * org.ow2.jasmine.vmm.agent.domain.policy.VMPlacementPolicy#placeVM(org
     * .ow2.jasmine.vmm.api.ServerPoolMXBean,
     * org.ow2.jasmine.vmm.api.VMConfigSpec)
     */
    public HostMXBean placeVM(final ServerPoolMXBean serverPool, final VMConfigSpec vmSpec) {
        List<HostMXBean> hosts = serverPool.getManagedHosts();

        List<HostMXBean> candidateHostList = new ArrayList<HostMXBean>();
        for (HostMXBean host : hosts) {
            if (host.getFreeMemoryMB() >= vmSpec.getMemorySizeMB()) {
                if (host.getVMImageStore().getFreeSpaceMB() > vmSpec.getDiskSizeMB()) {
                    candidateHostList.add(host);
                }
            }
        }
        Collections.sort(candidateHostList, new Comparator<HostMXBean>() {
            public int compare(final HostMXBean h1, final HostMXBean h2) {
                long freeMem1 = h1.getFreeMemoryMB();
                long freeMem2 = h2.getFreeMemoryMB();
                return (int) (freeMem1 - freeMem2);
            }
        });

        PackingVMPlacementPolicy.logger.debug("Packing VM policy selected host: "
            + (candidateHostList.size() > 0 ? candidateHostList.get(0).getHostName() : "none"));

        return candidateHostList.size() > 0 ? candidateHostList.get(0) : null;
    }

}

==> vmmagent-0.6.4-sources.jar: org/ow2/jasmine/vmm/agent/domain/policy/PackingVMPlacementPolicy.java <==
/**
 * JASMINe VMMapi: JASMINe Virtual Machine Management API
 * Copyright (C) 2009 France Telecom R&D
 * Contact: jasmine@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: PackingVMPlacementPolicy.java 6100 2010-02-23 14:05:27Z dangtran $
 * --------------------------------------------------------------------------
 */
package org.ow2.jasmine.vmm.agent.domain.policy;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;

import org.apache.log4j.Logger;
import org.ow2.jasmine.vmm.api.HostMXBean;
import org.ow2.jasmine.vmm.api.ServerPoolMXBean;
import org.ow2.jasmine.vmm.api.VMConfigSpec;
import org.ow2.jasmine.vmm.api.VMMException;

/**
 * PackingVMPlacementPolicy is a VM placement policy which attempts to pack the
 * virtual machines of a server pool on as few as physical hosts as possible.
 */
public class PackingVMPlacementPolicy implements VMPlacementPolicy {
    static Logger logger = Logger.getLogger(PackingVMPlacementPolicy.class);

    public PackingVMPlacementPolicy() {
    }

    /*
     * (non-Javadoc)
     * @see
     * org.ow2.jasmine.vmm.agent.domain.policy.VMPlacementPolicy#placeVM(org
     * .ow2.jasmine.vmm.api.ServerPoolMXBean,
     * org.ow2.jasmine.vmm.api.VMConfigSpec)
     */
    public HostMXBean placeVM(final ServerPoolMXBean serverPool, final VMConfigSpec vmSpec) {
        List<HostMXBean> hosts = serverPool.getManagedHosts();

        List<HostMXBean> candidateHostList = new ArrayList<HostMXBean>();
        for (HostMXBean host : hosts) {
            try {
                if (host.getFreeMemoryMB() > vmSpec.getMemorySizeMB()) {
                    if (host.getVMImageStore().getFreeSpaceMB() > vmSpec.getDiskSizeMB()) {
                        candidateHostList.add(host);
                    }
                }
            } catch (VMMException ex) {
                PackingVMPlacementPolicy.logger.error(ex);
            }
        }
        Collections.sort(candidateHostList, new Comparator<HostMXBean>() {
            public int compare(final HostMXBean h1, final HostMXBean h2) {
                try {
                    long freeMem1 = h1.getFreeMemoryMB();
                    long freeMem2 = h2.getFreeMemoryMB();
                    return (int) (freeMem1 - freeMem2);
                } catch (VMMException ex) {
                    PackingVMPlacementPolicy.logger.error(ex);
                    return 0;
                }
            }
        });

        try {
            PackingVMPlacementPolicy.logger.debug("Packing VM policy selected host: "
                + (candidateHostList.size() > 0 ? candidateHostList.get(0).getHostName() : "none"));
        } catch (VMMException ex) {
        }

        return candidateHostList.size() > 0 ? candidateHostList.get(0) : null;
    }

}
